defmodule Frenzy.HTTP do
  require Logger
  @redirect_codes [301, 302]

  # @spec get(url :: String.t()) :: {:ok, HTTPoison.Response.t()} | {:error, String.()}
  def get(url) do
    case HTTPoison.get(url) do
      {:ok, %HTTPoison.Response{status_code: 200} = response} ->
        {:ok, response}

      {:ok, %HTTPoison.Response{status_code: status_code, headers: headers}}
      when status_code in @redirect_codes ->
        headers
        |> Enum.find(fn {name, _value} -> name == "Location" end)
        |> case do
          {"Location", new_url} ->
            Logger.debug("Got 301 redirect from #{url} to #{new_url}")
            get(new_url)

          _ ->
            {:error, "Missing Location header for redirect"}
        end

      {:ok, %HTTPoison.Response{status_code: 403}} ->
        {:error, "403 Forbidden"}

      {:ok, %HTTPoison.Response{status_code: 404}} ->
        {:error, "404 Not Found"}

      {:error, %HTTPoison.Error{reason: reason}} ->
        {:error, reason}
    end
  end
end
